/******************************************************************************************
 *
 * CHECK_HPJD.C
 *
 * Program: HP printer plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 08-14-1999
 *
 * Command line: CHECK_HPJD <ip_address> [community]
 *
 * Description:
 *
 * This plugin will attempt to check the status of an HP printer.  The printer must 
 * have a JetDirect card installed and TCP/IP protocol stack enabled.  This plugin has
 * only been tested on a few printers and may not work well on all models of JetDirect
 * cards.  Multiple port JetDirect devices must have an IP address assigned to each port
 * in order to be monitored.
 *
 * Dependencies:
 *
 * This plugin used the 'snmpget' command included with the UCD-SNMP package.  If you don't
 * have the package installed you will need to download it from http://ucd-snmp.ucdavis.edu
 * before you can use this plugin.
 *
 * Return Values:
 *
 * UNKNOWN	= The plugin could not read/process the output it recieved from the printer
 * OK		= Printer looks normal
 * WARNING	= Low toner, paper jam, intervention required, paper out, etc.
 * CRITICAL	= The printer could not be reached (it's probably turned off)
 *
 * Acknowledgements:
 *
 * The idea for the plugin (as well as some code) were taken from Jim Trocki's pinter
 * alert script in his "mon" utility, found at http://www.kernel.org/software/mon
 *
 * Notes:
 * 'JetDirect' is copyrighted by Hewlett-Packard.  HP, please don't sue me... :-)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define HPJD_LINE_STATUS		".1.3.6.1.4.1.11.2.3.9.1.1.2.1"
#define HPJD_PAPER_STATUS		".1.3.6.1.4.1.11.2.3.9.1.1.2.2"		/* not used in this first revision */
#define HPJD_INTERVENTION_REQUIRED	".1.3.6.1.4.1.11.2.3.9.1.1.2.3"
#define HPJD_GD_PERIPHERAL_ERROR	".1.3.6.1.4.1.11.2.3.9.1.1.2.6"		
#define HPJD_GD_PAPER_JAM		".1.3.6.1.4.1.11.2.3.9.1.1.2.8"
#define HPJD_GD_PAPER_OUT		".1.3.6.1.4.1.11.2.3.9.1.1.2.9"
#define HPJD_GD_TONER_LOW		".1.3.6.1.4.1.11.2.3.9.1.1.2.10"
#define HPJD_GD_STATUS_DISPLAY		".1.3.6.1.4.1.11.2.3.9.1.1.3"

#define ONLINE		0
#define OFFLINE		1

#define MAX_INPUT_BUFFER	512

#define TEMP_FILE_NAME	"check_hpjd.tmp"
char temp_file[MAX_INPUT_BUFFER]="";

int my_system(char *);



int main(int argc, char **argv){
	char command_line[1024];
	char query_string[512];
	int result;
	int line;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char address[MAX_INPUT_BUFFER];
	char community[MAX_INPUT_BUFFER];
	char display_message[MAX_INPUT_BUFFER];
	char error_message[MAX_INPUT_BUFFER];
	int line_status=ONLINE;
	int paper_status=0;
	int peripheral_error=0;
	int paper_out=0;
	int paper_jam=0;
	int toner_low=0;
	int intervention_required=0;
	char *temp_buffer;

	if(argc<2||argc>3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("HP printer plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 08-14-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <ip_address> [community]\n",argv[0]);
		printf("\n");
		printf("Note:\n");
		printf(" <ip_address>     = The IP address of the JetDirect card\n");
		printf(" [community]      = An optional community string used for SNMP communication\n");
		printf("                    with the JetDirect card.  The default is 'public'.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the IP address of the JetDirect device */
	strcpy(address,argv[1]);

	/* get the community name to use for SNMP communication */
	if(argc>=3)
		strcpy(community,argv[2]);
	else
		strcpy(community,"public");

	/* create the query string */
	sprintf(query_string,"%s.0 %s.0 %s.0 %s.0 %s.0 %s.0 %s.0 %s.0",HPJD_GD_STATUS_DISPLAY,HPJD_LINE_STATUS,HPJD_PAPER_STATUS,HPJD_GD_PERIPHERAL_ERROR,HPJD_GD_PAPER_OUT,HPJD_GD_PAPER_OUT,HPJD_GD_TONER_LOW,HPJD_INTERVENTION_REQUIRED);

	/* get the path/file name of the temp file to use */
	snprintf(temp_file,sizeof(temp_file),"%s/%s",DEFAULT_TEMP_FILE_PATH,TEMP_FILE_NAME);
	temp_file[sizeof(temp_file)-1]='\x0';

	/* run the command */
	sprintf(command_line,"%s -v 1 %s %s %s > %s 2> %s",PATH_TO_SNMPGET,address,community,query_string,temp_file,temp_file);
	my_system(command_line);

	result=STATE_OK;

	/* open the temp file */
	fp=fopen(temp_file,"r");
	if(fp==NULL){
		remove(temp_file);
		printf("Error: Could not open temp file for reading\n");
		return STATE_UNKNOWN;
	        }

	line=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		/* strip the newline character from the end of the input */
		if(input_buffer[strlen(input_buffer)-1]=='\n')
			input_buffer[strlen(input_buffer)-1]='\x0';

		line++;

		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"=");

		switch(line){

		case 1:
			/* the first line should contain the display panel message */
			if(temp_buffer!=NULL)
				strcpy(display_message,temp_buffer+1);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;

		case 2:
			/* the second line should contain the line status */
			if(temp_buffer!=NULL)
				line_status=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
			

		case 3:
			/* the third line should contain the paper status */
			if(temp_buffer!=NULL)
				paper_status=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
			

	
		case 4:
			/* the fourth line should contain the paper out status */
			if(temp_buffer!=NULL)
				paper_out=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
			
	
		case 5:
			/* the fifth line should contain the paper out status */
			if(temp_buffer!=NULL)
				paper_out=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
	
		case 6:
			/* the sixth line should contain the toner low status */
			if(temp_buffer!=NULL)
				toner_low=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
		
		case 7:
			/* the seventh line should contain the intervention required status */
			if(temp_buffer!=NULL)
				intervention_required=atoi(temp_buffer);
			else{
				result=STATE_UNKNOWN;
				strcpy(error_message,input_buffer);
			        }
			break;
			
		default:
		        break;
		        }

		/* break out of the read loop if we have encountered an error */
		if(result!=STATE_OK)
			break;
	        }

	/* close the temp file */
	fclose(fp);

	/* if there wasn't any output, display an error */
	if(line==0){
		result=STATE_UNKNOWN;
		strcpy(error_message,"Could not read plugin output\n");
	        }

	/* if we haven't encountered any read errors, let's check the printer status results... */
	if(result==STATE_OK){

		if(paper_jam){
			result=STATE_WARNING;
			strcpy(error_message,"Paper Jam");
		        }
		else if(paper_out){
			result=STATE_WARNING;
			strcpy(error_message,"Out of Paper");
		        }
		else if(line_status==OFFLINE){
			result=STATE_WARNING;
			strcpy(error_message,"Printer Offline");
		        }
		else if(peripheral_error){
			result=STATE_WARNING;
			strcpy(error_message,"Peripheral Error");
		        }
		else if(intervention_required){
			result=STATE_WARNING;
			strcpy(error_message,"Intervention Required");
		        }
		else if(toner_low){
			result=STATE_WARNING;
			strcpy(error_message,"Toner Low");
		        }
	        }

	if(result==STATE_OK)
		printf("Printer ok - (%s)\n",display_message);

	else if(result==STATE_UNKNOWN){

		printf("%s\n",error_message);

		/* if the printer could not be reached, escalate this to a critical error */
		if(strstr(error_message,"Timeout"))
		  result=STATE_CRITICAL;
	        }

	else if(result==STATE_WARNING)
		printf("%s (%s)\n",error_message,display_message);

	/* remove the temp file */
	remove(temp_file);

	return result;
        }



int my_system(char *command){
        int pid;
	int status;
	int result;
	extern int errno;
	extern char **environ;


	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1)
	        return STATE_UNKNOWN;  

	/* execute the command in the child process */
        if (pid==0){
		char *argv[4];
		argv[0]="sh";
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;
		execve("/bin/sh",argv,environ);
		exit(STATE_UNKNOWN);
	        }

	/* wait for the command to finish */
        do{
                if (waitpid(pid, &status, 0) == -1) {
		        if (errno != EINTR)
                                return STATE_UNKNOWN;
                        } 

		/* get the exit code returned from the program */
		result=WEXITSTATUS(status);

		/* check the bounds for the return code */
		if(result<0||result>2)
			return STATE_UNKNOWN;

		return result;
           } while(1);

        return STATE_OK;
        }

