/******************************************************************************************
 *
 * CHECK_HTTP.C
 *
 * Program: HTTP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-21-1999
 *
 * Command line: CHECK_HTTP <host_address> [-e expect] [-u url] [-p port] [-hn host_name] 
 *                          [-wt warn_time] [-ct crit_time] [-to to_sec] 
 *
 * Description:
 *
 * This plugin will attempt to open an HTTP connection with the host.  Successul connects 
 * return STATE_OK, refusals and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.  Successful connects, but incorrect reponse messages from the host 
 * result in STATE_WARNING return values.  If you are checking a virtual server that uses
 * "host headers" you must supply the FQDN (fully qualified domain name) as the
 * [host_name] argument. 
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <signal.h>

#define OK		0
#define ERROR		-1

#define TRUE		1
#define FALSE		0

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define DEFAULT_TIMEOUT	10	/* timeout after 10 seconds */

#define HTTP_PORT	80
#define HTTP_PROTO	"tcp"
#define HTTP_EXPECT	"HTTP/1."
#define HTTP_URL	"/"

#define MAX_INPUT_BUFFER	256

time_t start_time,end_time;
int server_port=HTTP_PORT;
char server_name[256];
char host_name[256];
char server_url[256]=HTTP_URL;
char server_expect[256]=HTTP_EXPECT;
int timeout=DEFAULT_TIMEOUT;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;

int my_connect(char *address,int port,char *protocol,int *sd);
int process_arguments(int,char **);
void alarm_handler(int);


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];
	char *status_line;

	result=process_arguments(argc,argv);

	if(result!=OK){

		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("HTTP plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-21-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-e expect] [-u url] [-p port] [-hn host_name] [-wt warn_time] [-ct crit_time] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [expect]    = String to expect in first line of server response - default is \"%s\"\n",HTTP_EXPECT);
		printf(" [url]       = Optional URL to GET - default is root document\n");
		printf(" [port]      = Optional port number to use - default is %d\n",HTTP_PORT);
		printf(" [host_name] = Optional host name argument to GET command - used for servers using host headers\n");
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_TIMEOUT);
		printf("\n");
		printf("This plugin attempts to contact the HTTP service on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }

	/* initialize alarm signal handling */
	signal(SIGALRM,alarm_handler);

	/* set socket timeout */
	alarm(timeout);

	time(&start_time);

	/* try to connect to the host at the given port number */
	result=my_connect(server_name,server_port,HTTP_PROTO,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* send the GET request */
		sprintf(buffer,"GET %s HTTP/1.0\n",server_url);
		result=send(sd,buffer,strlen(buffer),0);
		if(strcmp(host_name,"")){
			sprintf(buffer,"Host: %s\n",host_name);
			result=send(sd,buffer,strlen(buffer),0);
		        }
		sprintf(buffer,"\n");
		result=send(sd,buffer,strlen(buffer),0);

		/* watch for the HTTP connection string */
		result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* return a CRITICAL status if we couldn't read any data */
		if(result==-1){
			printf("No data received from host\n");
			result=STATE_CRITICAL;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(strstr(buffer,HTTP_EXPECT)==NULL){
				if(server_port==HTTP_PORT)
					printf("Invalid HTTP response received from host\n");
				else
					printf("Invalid HTTP response received from host on port %d\n",server_port);
			        }

			/* else we got the HTTP string, so check the return code */
			else{

				time(&end_time);

				result=STATE_OK;

				status_line=strtok(buffer,"\n");

				if(strstr(status_line,"200")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"201")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"202")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"203")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"204")!=NULL)
					result=STATE_OK;

				/* we should be checking the redirected page, but for simplicity we don't */
				else if(strstr(status_line,"301")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"302")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"303")!=NULL)
					result=STATE_OK;
				else if(strstr(status_line,"304")!=NULL)
					result=STATE_OK;

				/* client errors result in a warning state */
				else if(strstr(status_line,"400")!=NULL)
					result=STATE_WARNING;
				else if(strstr(status_line,"401")!=NULL)
					result=STATE_WARNING;
				else if(strstr(status_line,"402")!=NULL)
					result=STATE_WARNING;
				else if(strstr(status_line,"403")!=NULL)
					result=STATE_WARNING;
				else if(strstr(status_line,"404")!=NULL)
					result=STATE_WARNING;

				/* server errors result in a critical state */
				else if(strstr(status_line,"500")!=NULL)
					result=STATE_CRITICAL;
				else if(strstr(status_line,"501")!=NULL)
					result=STATE_CRITICAL;
				else if(strstr(status_line,"502")!=NULL)
					result=STATE_CRITICAL;
				else if(strstr(status_line,"503")!=NULL)
					result=STATE_CRITICAL;

				else result=STATE_UNKNOWN;

				if(result==STATE_OK){

					if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
						result=STATE_CRITICAL;
					else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
						result=STATE_WARNING;

					printf("HTTP %s - %d second response time\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time));
				        }
				else
					printf("%s\n",status_line);

			        }
		        }

		/* close the connection */
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



int process_arguments(int argc, char **argv){
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strcpy(server_name,argv[1]);

	for(x=3;x<=argc;x++){

		if(!strcmp(argv[x-1],"-e")){
			if(x<argc){
				strcpy(server_expect,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-u")){
			if(x<argc){
				strcpy(server_url,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-hn")){
			if(x<argc){
				strcpy(host_name,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				timeout=atoi(argv[x]);
				if(timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }


int my_connect(char *host_name,int port,char *protocol,int *sd){
	struct sockaddr_in servaddr;
	struct hostent *hp;
	struct protoent *ptrp;
	int result;

	bzero((char *)&servaddr,sizeof(servaddr));
	servaddr.sin_family=AF_INET;
	servaddr.sin_port=htons(port);

	/* try to bypass using a DNS lookup if this is just an IP address */
	if(!inet_aton(host_name,&servaddr.sin_addr)){

		/* else do a DNS lookup */
		hp=gethostbyname((const char *)host_name);
		if(hp==NULL){
			printf("Invalid host name '%s'\n",host_name);
			return STATE_UNKNOWN;
		        }

		memcpy(&servaddr.sin_addr,hp->h_addr,hp->h_length);
	        }

	/* map TCP transport protocol name to protocol number */
	if(((int)(ptrp=getprotobyname(protocol)))==(int)0){
		printf("Cannot map \"%s\" to protocol number\n",protocol);
		return STATE_UNKNOWN;
	        }


	/* create a socket */
	*sd=socket(PF_INET,SOCK_STREAM,ptrp->p_proto);
	if(*sd<0){
		printf("Socket creation failed\n");
		return STATE_UNKNOWN;
	        }

	/* open a connection */
	time(&start_time);
	result=connect(*sd,(struct sockaddr *)&servaddr,sizeof(servaddr));
	if(result<0){
		switch(errno){  
		case ECONNREFUSED:
			if(port==HTTP_PORT)
				printf("Connection refused by host\n");
			else
				printf("Connection refused on port %d\n",port);
			break;
		case ETIMEDOUT:
			printf("Timeout while attempting connection\n");
			break;
		case ENETUNREACH:
			printf("Network is unreachable\n");
			break;
		default:
			printf("Connection refused or timed out\n");
		        }

		return STATE_CRITICAL;
	        }

	return STATE_OK;
        }


void alarm_handler(int sig){

	printf("Socket timeout after %d seconds\n",timeout);

	exit(STATE_CRITICAL);
        }

