/************************************************************************************************************
 *
 * CHECK_POP.C
 *
 * Program: POP3 plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-21-1999
 *
 * Command line: CHECK_POP <host_address> [-e expect] [-p port] [-to to_sec] [-wt warn_time] [-ct crit_time]
 *
 * Description:
 *
 * This plugin will attempt to open an POP connection with the host.  Successul connects 
 * return STATE_OK, refusals and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.  Successful connects, but incorrect reponse messages from the host 
 * result in STATE_WARNING return values.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***********************************************************************************************************/

#include "../common/config.h"
#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <signal.h>

#define OK		0
#define ERROR		-1

#define TRUE		1
#define FALSE		0

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define POP_PORT	110
#define POP_PROTO	"tcp"
#define POP_EXPECT     "+OK"
#define POP_QUIT	"QUIT\n"

#define DEFAULT_TIMEOUT	10	/* timeout after 10 seconds */

#define MAX_INPUT_BUFFER	256

time_t start_time,end_time;
int server_port=POP_PORT;
char server_name[256];
char server_expect[256]=POP_EXPECT;
int timeout=DEFAULT_TIMEOUT;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;

int my_connect(char *address,int port,char *protocol,int *sd);
int process_arguments(int,char **);
void alarm_handler(int);


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];

	result=process_arguments(argc,argv);

	if(result!=OK){

		printf("Invalid command arguments supplied\n");
		printf("\n");
		printf("POP3 plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-21-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-e expect] [-p port] [-wt warn_time] [-ct crit_time] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [expect]    = String to expect in first line of server response - default is \"%s\"\n",POP_EXPECT);
		printf(" [port]      = Optional port number to use - default is %d\n",POP_PORT);
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to contact the POP3 service on the specified host.\n");
		printf("If possible, supply and IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }

	/* initialize alarm signal handling */
	signal(SIGALRM,alarm_handler);

	/* set socket timeout */
	alarm(timeout);

	time(&start_time);

	/* try to connect to the host at the given port number */
	result=my_connect(server_name,server_port,POP_PROTO,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the SMTP connection string */
		result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(strstr(buffer,server_expect)==NULL){
				if(server_port==POP_PORT)
					printf("Invalid POP response received from host\n");
				else
					printf("Invalid POP response received from host on port %d\n",server_port);
				result=STATE_WARNING;
			        }

			else{
				time(&end_time);

				result=STATE_OK;

				if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
					result=STATE_CRITICAL;
				else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
					result=STATE_WARNING;

				printf("POP %s - %d second response time\n",(result==STATE_OK)?"ok":"problem",(int)(end_time-start_time));
			        }
		        }

		/* close the connection */
		send(sd,POP_QUIT,strlen(POP_QUIT),0);
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



int process_arguments(int argc, char **argv){
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strcpy(server_name,argv[1]);

	for(x=3;x<=argc;x++){

		if(!strcmp(argv[x-1],"-e")){
			if(x<argc){
				strcpy(server_expect,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				timeout=atoi(argv[x]);
				if(timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }



int my_connect(char *host_name,int port,char *protocol,int *sd){
	struct sockaddr_in servaddr;
	struct hostent *hp;
	struct protoent *ptrp;
	int result;

	bzero((char *)&servaddr,sizeof(servaddr));
	servaddr.sin_family=AF_INET;
	servaddr.sin_port=htons(port);

	/* try to bypass using a DNS lookup if this is just an IP address */
	if(!inet_aton(host_name,&servaddr.sin_addr)){

		/* else do a DNS lookup */
		hp=gethostbyname((const char *)host_name);
		if(hp==NULL){
			printf("Invalid host name '%s'\n",host_name);
			return STATE_UNKNOWN;
		        }

		memcpy(&servaddr.sin_addr,hp->h_addr,hp->h_length);
	        }

	/* map TCP transport protocol name to protocol number */
	if(((int)(ptrp=getprotobyname(protocol)))==(int)0){
		printf("Cannot map \"%s\" to protocol number\n",protocol);
		return STATE_UNKNOWN;
	        }

	/* create a socket */
	*sd=socket(PF_INET,SOCK_STREAM,ptrp->p_proto);
	if(*sd<0){
		printf("Socket creation failed\n");
		return STATE_UNKNOWN;
	        }

	/* open a connection */
	time(&start_time);
	result=connect(*sd,(struct sockaddr *)&servaddr,sizeof(servaddr));
	if(result<0){
		switch(errno){  
		case ECONNREFUSED:
			if(port==POP_PORT)
				printf("Connection refused by host\n");
			else
				printf("Connection refused at port %d by host\n",port);
			break;
		case ETIMEDOUT:
			printf("Timeout while attempting connection\n");
			break;
		case ENETUNREACH:
			printf("Network is unreachable\n");
			break;
		default:
			printf("Connection refused or timed out\n");
		        }

		return STATE_CRITICAL;
	        }

	return STATE_OK;
        }


void alarm_handler(int sig){

	printf("Socket timeout after %d seconds\n",timeout);

	exit(STATE_CRITICAL);
        }

